"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliInteractive = void 0;
const chalk = require("chalk");
const moment = require("moment");
const ora = require("ora");
const os_1 = require("os");
const common_1 = require("../../../core/common");
const socket_1 = require("../../../core/socket");
class CliInteractive {
    constructor(config) {
        this.config = config;
    }
    listen(server, client) {
        return __awaiter(this, void 0, void 0, function* () {
            const spinner = ora('Connecting').start();
            const stopSpinner = (delay) => new Promise(resolve => setTimeout(() => resolve(spinner.stop()), delay));
            try {
                yield client.connect();
                yield stopSpinner(500);
            }
            catch (err) {
                spinner.color = 'red';
                yield stopSpinner(500);
                throw err;
            }
            const address = yield server
                .listen(this.config.apiHost, this.config.apiPort);
            console.log();
            console.log(`${chalk.bold('API server:')} ${address}`);
            console.log(`${chalk.bold('Swagger UI:')} ${address}/swagger`);
            console.log();
            setTimeout(() => {
                client.store
                    .listen()
                    .subscribe(event => console.log(this.parseEvent(event)));
            }, 1000);
        });
    }
    parseEvent(event) {
        const eventName = {
            [socket_1.SocketEventType.ReceivedMessage]: this.config.emoji ? '📥 Message received' : 'Message received',
            [socket_1.SocketEventType.SentMessage]: this.config.emoji ? '📤 Message sent' : 'Message sent',
            [socket_1.SocketEventType.Reconnecting]: this.config.emoji ? '🔁 Retry connection' : 'Retry connection',
            [socket_1.SocketEventType.Connected]: this.config.emoji ? '🔄 Connection established' : 'Connection established',
            [socket_1.SocketEventType.Error]: this.config.emoji ? '❗️ Error' : 'Error',
            [socket_1.SocketEventType.Closed]: this.config.emoji ? '🚫️ Connection closed' : 'Connection closed',
        };
        const id = chalk.bold(event.id.toString().padStart(5, '0'));
        const date = moment(event.date).format('YYYY-MM-D HH:mm:ss');
        const title = `${id} [${date}] ${eventName[event.type]}`;
        const isConnectionEvent = event.type === socket_1.SocketEventType.Connected || event.type === socket_1.SocketEventType.Reconnecting;
        const message = isConnectionEvent
            ? { type: event.message.type, address: event.message.address }
            : event.message;
        const options = isConnectionEvent
            ? { maxStringLength: 200 }
            : undefined;
        const humanizedMessage = common_1.MessageUtil
            .humanize(message, options)
            .replace(new RegExp(os_1.EOL, 'g'), `${os_1.EOL}  `);
        const body = '  ' + humanizedMessage + os_1.EOL;
        return `${title}:${os_1.EOL}${body}`;
    }
}
exports.CliInteractive = CliInteractive;
