"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpController = void 0;
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const http_schema_1 = require("../../http/nest/http.schema");
const common_2 = require("../../../core/common");
const socket_1 = require("../../../core/socket");
let HttpController = class HttpController {
    constructor(client) {
        this.client = client;
    }
    getMessage(id) {
        const message = this.client
            .store
            .list()
            .find(m => m.id === Number(id));
        if (!message) {
            throw new common_1.NotFoundException(`message with id ${id} not found`, 'Not Found');
        }
        return message;
    }
    getMessageList(query) {
        return this.client
            .store
            .list({ types: query.type });
    }
    sendMessage(body) {
        const data = body.data;
        const bytes = body.bytes;
        const event = body.event;
        if (data !== undefined && bytes !== undefined) {
            throw new common_1.BadRequestException('body must match exactly one schema in oneOf', 'Bad Request');
        }
        if (data !== undefined) {
            return this.client.sendData(data, event);
        }
        return this.client.sendBytes(bytes, event);
    }
};
__decorate([
    (0, common_1.Get)('/messages/:id'),
    (0, swagger_1.ApiTags)('Messages'),
    (0, swagger_1.ApiOperation)({ description: 'Get message by id', summary: 'Get message by id' }),
    (0, swagger_1.ApiOkResponse)({ type: http_schema_1.MessageModel }),
    (0, swagger_1.ApiBadRequestResponse)({ type: http_schema_1.ErrorModel }),
    (0, swagger_1.ApiNotFoundResponse)({ type: http_schema_1.ErrorModel }),
    (0, swagger_1.ApiInternalServerErrorResponse)({ type: http_schema_1.ErrorModel }),
    __param(0, (0, common_1.Param)('id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [String]),
    __metadata("design:returntype", Object)
], HttpController.prototype, "getMessage", null);
__decorate([
    (0, common_1.Get)('/messages'),
    (0, swagger_1.ApiTags)('Messages'),
    (0, swagger_1.ApiOperation)({ description: 'Get message list', summary: 'Get message list' }),
    (0, swagger_1.ApiQuery)({ name: 'type', enum: common_2.EnumUtil.values(socket_1.SocketEventType), isArray: true, required: false }),
    (0, swagger_1.ApiOkResponse)({ type: http_schema_1.MessageModel, isArray: true }),
    (0, swagger_1.ApiBadRequestResponse)({ type: http_schema_1.ErrorModel }),
    (0, swagger_1.ApiNotFoundResponse)({ type: http_schema_1.ErrorModel }),
    (0, swagger_1.ApiInternalServerErrorResponse)({ type: http_schema_1.ErrorModel }),
    __param(0, (0, common_1.Query)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [http_schema_1.GetMessageListQuery]),
    __metadata("design:returntype", Array)
], HttpController.prototype, "getMessageList", null);
__decorate([
    (0, common_1.Post)('/messages'),
    (0, swagger_1.ApiTags)('Messages'),
    (0, swagger_1.ApiOperation)({ description: 'Send message', summary: 'Send message' }),
    (0, swagger_1.ApiExtraModels)(http_schema_1.SendDataMessageBody, http_schema_1.SendBytesMessageBody),
    (0, swagger_1.ApiBody)({ schema: { oneOf: (0, swagger_1.refs)(http_schema_1.SendDataMessageBody, http_schema_1.SendBytesMessageBody) } }),
    (0, swagger_1.ApiCreatedResponse)({ type: http_schema_1.SendMessageResponse }),
    (0, swagger_1.ApiBadRequestResponse)({ type: http_schema_1.ErrorModel }),
    (0, swagger_1.ApiNotFoundResponse)({ type: http_schema_1.ErrorModel }),
    (0, swagger_1.ApiInternalServerErrorResponse)({ type: http_schema_1.ErrorModel }),
    __param(0, (0, common_1.Body)()),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [http_schema_1.SendDataBody]),
    __metadata("design:returntype", Promise)
], HttpController.prototype, "sendMessage", null);
HttpController = __decorate([
    (0, common_1.Controller)(),
    __param(0, (0, common_1.Inject)('SOCKET_CLIENT')),
    __metadata("design:paramtypes", [Object])
], HttpController);
exports.HttpController = HttpController;
