"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HttpServer = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const swagger_1 = require("@nestjs/swagger");
const http_filter_1 = require("../../http/nest/http.filter");
const http_module_1 = require("../../http/nest/http.module");
class HttpServer {
    constructor(client, version, app) {
        this.client = client;
        this.version = version;
        this.app = app;
    }
    listen(host, port) {
        return __awaiter(this, void 0, void 0, function* () {
            this.app = yield this.init();
            yield this.app.listen(port, host);
            return this.app.getUrl();
        });
    }
    engine() {
        return this.app;
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            const app = yield core_1.NestFactory.create(http_module_1.HttpModule.forRoot(this.client), {
                logger: false
            });
            app.useGlobalPipes(new common_1.ValidationPipe({ transform: true }));
            app.useGlobalFilters(new http_filter_1.HttpFilter());
            app.enableCors();
            const swaggerConfig = new swagger_1.DocumentBuilder()
                .setTitle('iola')
                .setDescription('API documentation for iola client')
                .setVersion(this.version)
                .build();
            const swaggerDocument = swagger_1.SwaggerModule
                .createDocument(app, swaggerConfig);
            swagger_1.SwaggerModule
                .setup('swagger', app, swaggerDocument);
            return app;
        });
    }
}
exports.HttpServer = HttpServer;
