"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BinaryStore = void 0;
const rxjs_1 = require("rxjs");
class BinaryStore {
    constructor() {
        this.chunks = [];
        this.buffer$ = new rxjs_1.ReplaySubject(100);
        this.timer = setInterval(() => this.add(Buffer.alloc(0)), 1000);
    }
    listen() {
        return this.buffer$;
    }
    add(chunk) {
        if (this.chunks.length === 0) {
            return this.pushIfNecessary(chunk);
        }
        const last = this.chunks[this.chunks.length - 1];
        if (Date.now() - last.date > 100) {
            const buffer = Buffer.concat(this.chunks.map(ch => ch.chunk));
            this.chunks = [];
            this.buffer$.next(buffer);
        }
        this.pushIfNecessary(chunk);
    }
    close() {
        clearInterval(this.timer);
        this.clear();
    }
    clear() {
        this.chunks = [];
        this.buffer$ = new rxjs_1.ReplaySubject(100);
    }
    pushIfNecessary(chunk) {
        if (chunk.length > 0) {
            this.chunks.push({ date: Date.now(), chunk: chunk });
        }
    }
}
exports.BinaryStore = BinaryStore;
