#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const cli_1 = require("./api/cli");
const http_1 = require("./api/http");
const socket_1 = require("./core/socket");
(() => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const version = '1.0.0';
        const config = cli_1.CliFactory
            .createParser(version)
            .parse();
        detectSwaggerUIAssets();
        const client = socket_1.SocketFactory.createClient({
            type: config.socketType,
            address: config.socketAddress,
            binaryEncoding: config.binaryEncoding,
            connectionTimeout: config.connectionTimeout,
            reconnectionInterval: config.reconnectionInterval,
            headers: config.headers,
            ioAuth: config.ioAuth,
            ioTransport: config.ioTransport,
            netSync: config.netSync,
            replyTimeout: config.replyTimeout,
        });
        process.on('uncaughtException', error => client.store.add({
            type: socket_1.SocketEventType.Error,
            date: new Date(),
            message: {
                uncaughtException: error.message
            },
        }));
        process.on('unhandledRejection', error => client.store.add({
            type: socket_1.SocketEventType.Error,
            date: new Date(),
            message: {
                uncaughtException: error
            },
        }));
        const server = http_1.HttpFactory
            .createServer(client, version);
        yield cli_1.CliFactory
            .createInteractive(config)
            .listen(server, client);
    }
    catch (error) {
        const message = error instanceof Error
            ? error.message
            : JSON.stringify(error);
        console.error(message);
        process.exit(1);
    }
}))();
function detectSwaggerUIAssets() {
    path.join(__dirname, '../node_modules/swagger-ui-dist/swagger-ui.css');
    path.join(__dirname, '../node_modules/swagger-ui-dist/favicon-16x16.png');
    path.join(__dirname, '../node_modules/swagger-ui-dist/favicon-32x32.png');
}
